import { Injectable, Component, Output, EventEmitter, ViewChild, NgModule } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const terraformerWktParser = require('terraformer-wkt-parser');
class GeocoderService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        this.geocoderBaseUrl = 'https://geodata.nationaalgeoregister.nl/locatieserver/v3';
    }
    /**
     * @param {?} query
     * @param {?=} options
     * @return {?}
     */
    suggest(query, options) {
        let /** @type {?} */ fq = '*';
        let /** @type {?} */ start = '0';
        let /** @type {?} */ rows = '10';
        if (options) {
            fq = options.fq || fq;
            start = options.start || start;
            rows = options.rows || rows;
        }
        return this.http.get(`${this.geocoderBaseUrl}/suggest?`, {
            params: {
                q: query,
                fq: fq,
                // Also get percelen
                start,
                rows
            }
        }).toPromise().then((suggestResultObject) => {
            const /** @type {?} */ collations = this.parseCollations(suggestResultObject.spellcheck.collations);
            const /** @type {?} */ places = this.formatPlaces(suggestResultObject);
            return { collations, places };
        });
    }
    /**
     * @param {?} id
     * @return {?}
     */
    lookup(id) {
        return this.http.get(`${this.geocoderBaseUrl}/lookup?`, {
            params: {
                id: id,
                fl: '*',
            }
        }).toPromise().then((lookupResultObject) => {
            return this.formatLookupResponse(lookupResultObject);
        });
    }
    /**
     * @param {?} searchTerm
     * @param {?=} options
     * @return {?}
     */
    free(searchTerm, options) {
        let /** @type {?} */ fq = '*';
        let /** @type {?} */ fl = '*';
        let /** @type {?} */ start = '0';
        let /** @type {?} */ rows = '10';
        if (options) {
            fq = options.fq || fq;
            fl = options.fl || fl;
            start = options.start || start;
            rows = options.rows || rows;
        }
        return this.http.get(`${this.geocoderBaseUrl}/free?`, {
            params: {
                q: searchTerm,
                fl,
                fq,
                rows
            },
        }).toPromise().then((freeResultObject) => {
            return this.formatLookupResponse(freeResultObject);
        });
    }
    /**
     * @param {?} collations
     * @return {?}
     */
    parseCollations(collations) {
        const /** @type {?} */ parsedCollations = [];
        for (let /** @type {?} */ i = 0; i < collations.length; i += 2) {
            const /** @type {?} */ collation = {
                id: i,
                naam: collations[i + 1].misspellingsAndCorrections[1],
                weergavenaam: `${collations[i + 1].hits} resultaten gevonden voor <strong> ${collations[i + 1].misspellingsAndCorrections[1]} </strong>`,
                hits: collations[i + 1].hits,
            };
            parsedCollations.push(collation);
        }
        return parsedCollations;
    }
    /**
     * @param {?} suggestResultObject
     * @return {?}
     */
    formatPlaces(suggestResultObject) {
        const /** @type {?} */ places = suggestResultObject.response.docs.map((place) => {
            return {
                id: place.id,
                type: place.type,
                weergavenaam: place.weergavenaam,
                score: place.score,
                highlight: suggestResultObject.highlighting[place.id].suggest[0]
            };
        });
        return places;
    }
    /**
     * Parse WKT in lookup response.
     * @param {?} lookupResultObject
     * @return {?}
     */
    formatLookupResponse(lookupResultObject) {
        const /** @type {?} */ formatted = lookupResultObject.response.docs.map((lookupResult) => {
            const /** @type {?} */ formattedLookupResult = lookupResult;
            formattedLookupResult.centroide_ll = terraformerWktParser.parse(lookupResult.centroide_ll);
            formattedLookupResult.centroide_rd = terraformerWktParser.parse(lookupResult.centroide_rd);
            formattedLookupResult.geometrie_rd = terraformerWktParser.parse(lookupResult.geometrie_rd);
            formattedLookupResult.geometrie_ll = terraformerWktParser.parse(lookupResult.geometrie_ll);
            // formattedLookupResult.bbox_rd = (new terraformer.Primitive(formattedLookupResult.geometrie_rd) as any).bbox();
            // formattedLookupResult.bbox_ll = (new terraformer.Primitive(formattedLookupResult.geometrie_ll) as any).bbox();
            return formattedLookupResult;
        });
        return formatted[0] || {};
    }
}
GeocoderService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
GeocoderService.ctorParameters = () => [
    { type: HttpClient, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const L = require('leaflet');
class GeocoderComponent {
    /**
     * @param {?} geocoderService
     */
    constructor(geocoderService) {
        this.geocoderService = geocoderService;
        this.placeFound = new EventEmitter();
        this.searchInput = '';
        this.places = [];
        this.collations = [];
        this.searchThreshold = 2;
        this.foundPlace = null;
        this.selectedItem = [];
        this.selectedIndex = -1;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        L.DomEvent.disableClickPropagation(this.geocoderRef.nativeElement);
    }
    /**
     * @param {?} id
     * @return {?}
     */
    lookup(id) {
        this.geocoderService.lookup(id).then((lookupObj) => {
            this.foundPlace = lookupObj;
            this.fillInput(this.foundPlace.weergavenaam);
            this.placeFound.emit(this.foundPlace);
            this.clearPlaces();
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    suggest(event) {
        if (event && event.code && event.code === 'Enter') {
            return;
        }
        if (this.searchInput.length > this.searchThreshold) {
            this.geocoderService.suggest(this.searchInput).then((suggestResponse) => {
                this.places = suggestResponse.places;
                this.collations = suggestResponse.collations;
            });
        }
        else {
            this.clearPlaces();
        }
    }
    /**
     * @return {?}
     */
    clear() {
        this.foundPlace = null;
        this.searchInput = '';
        this.clearPlaces();
    }
    /**
     * @return {?}
     */
    clearPlaces() {
        this.resetIndex();
        this.places = [];
        this.collations = [];
    }
    /**
     * @return {?}
     */
    handleEnter() {
        if (this.selectedIndex === -1) {
            return;
        }
        const /** @type {?} */ selectedPlace = this.places[this.selectedIndex];
        this.lookup(selectedPlace.id);
    }
    /**
     * @return {?}
     */
    isNoResultsFound() {
        const /** @type {?} */ reachedThreshold = this.searchInput.length > this.searchThreshold;
        const /** @type {?} */ noSuggestions = this.places.length === 0;
        const /** @type {?} */ noResult = (this.foundPlace == null);
        return (reachedThreshold) && (noSuggestions) && (noResult);
    }
    /**
     * @return {?}
     */
    showCollations() {
        return this.collations.length > 0 && this.places.length === 0;
    }
    /**
     * @param {?} content
     * @return {?}
     */
    fillInput(content) {
        this.searchInput = content;
    }
    /**
     * @param {?} i
     * @return {?}
     */
    isHighlighted(i) {
        if (i === this.selectedIndex) {
            return true;
        }
    }
    /**
     * @return {?}
     */
    moveUp() {
        if (this.selectedIndex > 0) {
            this.selectedIndex--;
        }
    }
    /**
     * @return {?}
     */
    moveDown() {
        if (this.selectedIndex < this.places.length) {
            this.selectedIndex++;
        }
    }
    /**
     * @return {?}
     */
    resetIndex() {
        this.selectedIndex = -1;
    }
    /**
     * @return {?}
     */
    canQuery() {
        if (this.searchInput.length > this.searchThreshold) {
            return true;
        }
    }
    /**
     * @return {?}
     */
    canClear() {
        if (this.searchInput.length > 0) {
            return true;
        }
    }
}
GeocoderComponent.decorators = [
    { type: Component, args: [{
                styles: [`::-webkit-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:-moz-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}::-moz-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:-ms-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}::-ms-input-placeholder{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:placeholder-shown{font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}:focus{outline:0}input,li,p,span{margin:0;font-family:Verdana,Geneva,sans-serif;font-size:14px;color:#797979}.geocoder-container{width:100%;position:relative}.geocoder-search-container{width:100%;border:none;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);background:#fff}.geocoder-search-input{padding:15px;border:none;width:262px;overflow:hidden}button{position:absolute;cursor:pointer}.searchButton{top:13px;right:50px;padding-right:15px;background:0 0;border:none;border-right:1px solid #797979;height:20px}.closeButton{top:10px;right:10px;background:0 0;border:none;height:26px}svg{width:18px;color:#797979}.closeButton:hover svg,.highlight svg,.searchButton:hover svg{color:#79abff;-webkit-transition:all .1s cubic-bezier(.075,.82,.165,1);transition:all .1s cubic-bezier(.075,.82,.165,1)}.geocoder-suggestions-results{top:53px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%}.geocoder-collations-list,.geocoder-suggestions-list{list-style-type:none;padding:0;margin:0}.geocoder-suggestion{padding:15px;position:relative;border-bottom:1px solid #f1f1f1}.geocoder-suggestion-title{max-width:243px}.geocoder-suggestion-type{text-transform:uppercase;font-weight:700;font-size:11px;position:absolute;right:15px;top:36%;color:#bfbfbf}.geocoder-collation:hover,.geocoder-suggestion:hover{cursor:pointer;background-color:#f7f7f7}.focus-background{background-color:#f7f7f7}.geocoder-no-results{top:53px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%;width:calc(100% - 30px);padding:15px}.geocoder-collation-container{top:108px;position:absolute;background:#fff;-webkit-box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);box-shadow:0 3px 6px rgba(0,0,0,.16),0 3px 6px rgba(0,0,0,.23);width:100%}.geocoder-collation-container li{padding:15px;border-bottom:1px solid #f1f1f1}`],
                template: `<div  #geocoder
      (keydown.ArrowUp)="moveUp()"
      (keydown.ArrowDown)="moveDown()"
      (keydown.Enter)="handleEnter()"
      class="geocoder-container"
      >
  <div class="geocoder-search-container">
    <input class="geocoder-search-input"
          placeholder="Zoek op adres, postcode of perceel..."
          [(ngModel)]="searchInput"
          (keyup)="suggest($event)"
          (focus)="suggest($event)"
          type="text"
          maxlength="100">
    <button class="searchButton" (click)="suggest($event)" [ngClass]="{'highlight': canQuery()}"> <svg aria-hidden="true" data-fa-processed="" data-prefix="fal" data-icon="search" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" class="svg-inline--fa fa-search fa-w-16 fa-9x"><path fill="currentColor" d="M508.5 481.6l-129-129c-2.3-2.3-5.3-3.5-8.5-3.5h-10.3C395 312 416 262.5 416 208 416 93.1 322.9 0 208 0S0 93.1 0 208s93.1 208 208 208c54.5 0 104-21 141.1-55.2V371c0 3.2 1.3 6.2 3.5 8.5l129 129c4.7 4.7 12.3 4.7 17 0l9.9-9.9c4.7-4.7 4.7-12.3 0-17zM208 384c-97.3 0-176-78.7-176-176S110.7 32 208 32s176 78.7 176 176-78.7 176-176 176z" class=""></path></svg></button>
    <button class="closeButton" (click)="clear()" [ngClass]="{'highlight': canClear()}"> <svg aria-hidden="true" data-fa-processed="" data-prefix="fal" data-icon="times" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512" class="svg-inline--fa fa-times fa-w-12 fa-7x"><path fill="currentColor" d="M217.5 256l137.2-137.2c4.7-4.7 4.7-12.3 0-17l-8.5-8.5c-4.7-4.7-12.3-4.7-17 0L192 230.5 54.8 93.4c-4.7-4.7-12.3-4.7-17 0l-8.5 8.5c-4.7 4.7-4.7 12.3 0 17L166.5 256 29.4 393.2c-4.7 4.7-4.7 12.3 0 17l8.5 8.5c4.7 4.7 12.3 4.7 17 0L192 281.5l137.2 137.2c4.7 4.7 12.3 4.7 17 0l8.5-8.5c4.7-4.7 4.7-12.3 0-17L217.5 256z" class=""></path></svg></button>
  </div>
  <div class="geocoder-suggestions-container">
    <div class="geocoder-suggestions-results" *ngIf="places.length > 0">
      <ul class="geocoder-suggestions-list">
        <li class="geocoder-suggestion"
            tabindex={{i}}
            *ngFor="let place of places; let i = index;"
            (click)="lookup(place.id)"
            [ngClass]="{'focus-background': isHighlighted(i)}">
            <p class="geocoder-suggestion-title" [innerHTML]="place.highlight"></p>
            <p class="geocoder-suggestion-type"> {{place.type}} </p>
        </li>
      </ul>
    </div>
    <div class="geocoder-no-results" *ngIf="isNoResultsFound()">
      <p>Geen zoekresultaten gevonden voor <strong> {{this.searchInput}}</strong>.</p>
    </div>
    <div class="geocoder-collation-container" *ngIf="showCollations()">
      <ul class="geocoder-collations-list">
        <li class="geocoder-collation"
            tabindex={{i}}
            (click)="fillInput(collation.naam);suggest()"
            *ngFor="let collation of collations; let i = index;"
            [innerHTML]="collation.weergavenaam">
        </li>
      </ul>
    </div>
  </div>
</div>
`,
                selector: 'geocoder'
            },] },
];
/** @nocollapse */
GeocoderComponent.ctorParameters = () => [
    { type: GeocoderService, },
];
GeocoderComponent.propDecorators = {
    "geocoderRef": [{ type: ViewChild, args: ['geocoder',] },],
    "placeFound": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GeocoderModule {
}
GeocoderModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule, FormsModule, CommonModule],
                exports: [GeocoderComponent],
                declarations: [GeocoderComponent],
                providers: [GeocoderService]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { GeocoderModule, GeocoderService, GeocoderComponent as ɵa };
//# sourceMappingURL=angular-geocoder.js.map
